/*
 *  sunsaverEEPROM.c - This program reads all the EEPROM registers on a Moringstar SunSaver Duo and prints the results.
 *  

Copyright 2014 Tom Rinehart.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


/* Compile with: cc `pkg-config --cflags --libs libmodbus` sunsaverEEPROM.c -o sunsaverEEPROM */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <modbus.h>

#include "powersystem.h"

int main(void)
{
	modbus_t *ctx;
	int rc;
	unsigned short mt_id, mb_id;
	float priority_off, priority_on, tcompcoef;
	float v_adsorption_b1d1off, v_equalize_b1d1off, v_float_b1d1off, v_reg_max_b1d1off;
	unsigned short t_float_b1d1off, t_equalize_b1d1off;
	float v_adsorption_b1d1on, v_equalize_b1d1on, v_float_b1d1on, v_reg_max_b1d1on;
	unsigned short t_float_b1d1on, t_equalize_b1d1on;
	float v_adsorption_b2d2off, v_equalize_b2d2off, v_float_b2d2off, v_reg_max_b2d2off;
	unsigned short t_float_b2d2off, t_equalize_b2d2off;
	float v_adsorption_b2d2on, v_equalize_b2d2on, v_float_b2d2on, v_reg_max_b2d2on;
	unsigned short t_float_b2d2on, t_equalize_b2d2on;
	uint16_t data[32];
	
	/* Set up a new MODBUS context */
	ctx = modbus_new_rtu(SERIALPORTPATH, 9600, 'N', 8, 2);
	if (ctx == NULL) {
		fprintf(stderr, "Unable to create the libmodbus context\n");
		return -1;
	}
	
	/* Set the slave id to the SunSaver Duo MODBUS id */
	modbus_set_slave(ctx, SUNSAVERDUO);
	
	/* Open the MODBUS connection to the SunSaver Duo */
    if (modbus_connect(ctx) == -1) {
        fprintf(stderr, "Connection failed: %s\n", modbus_strerror(errno));
        modbus_free(ctx);
        return -1;
    }
	
	/* Read the EEPROM Registers and convert the results to their proper values */
	rc = modbus_read_registers(ctx, 0xE000, 30, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	printf("\nEEPROM Registers\n\n");
	
	printf("Custom Configuration, Common Values\n\n");
	
	mt_id=data[0];
	printf("mt_id = %d\n",mt_id);
	
	mb_id=data[1];
	printf("mb_id = %d\n",mb_id);
	
	priority_off=data[2]*100.0/417.0;
	printf("priority_off = %.2f %%\n",priority_off);
	
	priority_on=data[3]*100.0/417.0;
	printf("priority_on = %.2f %%\n",priority_on);
	
	tcompcoef=data[4]/1800.0;
	printf("tcompcoef = %.2f V\n",tcompcoef);
	
	printf("\nCustom Configuration, Battery 1, DIP 1 OFF\n\n");
	
	v_adsorption_b1d1off=data[6]/1800.0;
	printf("v_adsorption_b1d1off = %.2f V\n",v_adsorption_b1d1off);
	
	v_equalize_b1d1off=data[7]/1800.0;
	printf("v_equalize_b1d1off = %.2f V\n",v_equalize_b1d1off);
	
	v_float_b1d1off=data[8]/1800.0;
	printf("v_float_b1d1off = %.2f V\n",v_float_b1d1off);
	
	v_reg_max_b1d1off=data[9]/1800.0;
	printf("v_reg_max_b1d1off = %.2f V\n",v_reg_max_b1d1off);
	
	t_float_b1d1off=data[10];
	printf("t_float_b1d1off = %d s\n",t_float_b1d1off);
	
	t_equalize_b1d1off=data[11];
	printf("t_equalize_b1d1off = %d s\n",t_equalize_b1d1off);
	
	printf("\nCustom Configuration, Battery 1, DIP 1 ON\n\n");
	
	v_adsorption_b1d1on=data[12]/1800.0;
	printf("v_adsorption_b1d1on = %.2f V\n",v_adsorption_b1d1on);
	
	v_equalize_b1d1on=data[13]/1800.0;
	printf("v_equalize_b1d1on = %.2f V\n",v_equalize_b1d1on);
	
	v_float_b1d1on=data[14]/1800.0;
	printf("v_float_b1d1on = %.2f V\n",v_float_b1d1on);
	
	v_reg_max_b1d1on=data[15]/1800.0;
	printf("v_reg_max_b1d1on = %.2f V\n",v_reg_max_b1d1on);
	
	t_float_b1d1on=data[16];
	printf("t_float_b1d1on = %d s\n",t_float_b1d1on);
	
	t_equalize_b1d1on=data[17];
	printf("t_equalize_b1d1on = %d s\n",t_equalize_b1d1on);
	
	printf("\nCustom Configuration, Battery 2, DIP 2 OFF\n\n");
	
	v_adsorption_b2d2off=data[18]/1800.0;
	printf("v_adsorption_b2d2off = %.2f V\n",v_adsorption_b2d2off);
	
	v_equalize_b2d2off=data[19]/1800.0;
	printf("v_equalize_b2d2off = %.2f V\n",v_equalize_b2d2off);
	
	v_float_b2d2off=data[20]/1800.0;
	printf("v_float_b2d2off = %.2f V\n",v_float_b2d2off);
	
	v_reg_max_b2d2off=data[21]/1800.0;
	printf("v_reg_max_b2d2off = %.2f V\n",v_reg_max_b2d2off);
	
	t_float_b2d2off=data[22];
	printf("t_float_b2d2off = %d s\n",t_float_b2d2off);
	
	t_equalize_b2d2off=data[23];
	printf("t_equalize_b2d2off = %d s\n",t_equalize_b2d2off);
	
	printf("\nCustom Configuration, Battery 2, DIP 2 ON\n\n");
	
	v_adsorption_b2d2on=data[24]/1800.0;
	printf("v_adsorption_b2d2on = %.2f V\n",v_adsorption_b2d2on);
	
	v_equalize_b2d2on=data[25]/1800.0;
	printf("v_equalize_b2d2on = %.2f V\n",v_equalize_b2d2on);
	
	v_float_b2d2on=data[26]/1800.0;
	printf("v_float_b2d2on = %.2f V\n",v_float_b2d2on);
	
	v_reg_max_b2d2on=data[27]/1800.0;
	printf("v_reg_max_b2d2on = %.2f V\n",v_reg_max_b2d2on);
	
	t_float_b2d2on=data[28];
	printf("t_float_b2d2on = %d s\n",t_float_b2d2on);
	
	t_equalize_b2d2on=data[29];
	printf("t_equalize_b2d2on = %d s\n",t_equalize_b2d2on);
	
	/* Close the MODBUS connection */
    modbus_close(ctx);
    modbus_free(ctx);
	
	return(0);
}

