/*
 *  sunsaverRAM.c - This program reads all the RAM registers on a Moringstar SunSaver Duo and prints the results.
 *  

Copyright 2014 Tom Rinehart.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


/* Compile with: cc `pkg-config --cflags --libs libmodbus` sunsaverRAM.c -o sunsaverRAM */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <modbus.h>

#include "powersystem.h"

int main(void)
{
	modbus_t *ctx;
	int rc;
	float vb1, vb2, va, ia1, ia2;
	float vref1, vref2, dutcyc1, dutcyc2, vmaxb1, vminb1, vmaxb2, vminb2, iamax;
	unsigned int hours;
	float ah_total, ah_b1, ah_b2;
	float tcompcoef, priority, vabs1, veql1, vfloat1, vclamp1, vabs2, veql2, vfloat2, vclamp2;
	short tfloat1, teq1, tfloat2, teq2;
	short ta_F, ta_C, tr_F, tr_C, ths_F, ths_C, dc1, dc2, state, faults, flags, DIPs;
	short b1_state, b2_state, b1_detect, b2_detect;
	uint16_t data[50];
	
	/* Set up a new MODBUS context */
	ctx = modbus_new_rtu(SERIALPORTPATH, 9600, 'N', 8, 2);
	if (ctx == NULL) {
		fprintf(stderr, "Unable to create the libmodbus context\n");
		return -1;
	}
	
	/* Set the slave id to the SunSaver Duo MODBUS id */
	modbus_set_slave(ctx, SUNSAVERDUO);
	
	/* Open the MODBUS connection to the SunSaver Duo */
    if (modbus_connect(ctx) == -1) {
        fprintf(stderr, "Connection failed: %s\n", modbus_strerror(errno));
        modbus_free(ctx);
        return -1;
    }
	
	/* Read the first 41 RAM Registers */
	rc = modbus_read_registers(ctx, 0x0000, 41, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	/* Convert the results to their proper values and print them out */
	printf("\nRAM Registers\n\n");
	
	vb1=data[0]/1800.0;
	printf("vb1=%.2f V\n",vb1);
	
	vb2=data[1]/1800.0;
	printf("vb2=%.2f V\n",vb2);
	
	va=data[2]/1032.0;
	printf("va=%.2f V\n",va);
	
	ia1=data[3]/673.0;
	printf("ia1=%.2f A\n",ia1);
	
	ia2=data[4]/673.0;
	printf("ia2=%.2f A\n",ia2);
	
	vref1=data[8]/1800.0;
	printf("vref1=%.2f V\n",vref1);
	
	vref2=data[9]/1800.0;
	printf("vref2=%.2f V\n",vref2);
	
	dutcyc1=data[10]*100.0/417.0;
	printf("dutcyc1=%.2f %%\n",dutcyc1);
	
	dutcyc2=data[11]*100.0/417.0;
	printf("dutcyc2=%.2f %%\n",dutcyc2);
	
	vmaxb1=data[12]/1800.0;
	printf("vmaxb1=%.2f V\n",vmaxb1);
	
	vminb1=data[13]/1800.0;
	printf("vminb1=%.2f V\n",vminb1);
	
	vmaxb2=data[14]/1800.0;
	printf("vmaxb2=%.2f V\n",vmaxb2);
	
	vminb2=data[15]/1800.0;
	printf("vminb2=%.2f V\n",vminb2);
		
	iamax=data[16]/673.0;
	printf("iamax=%.2f A\n",iamax);
	
	hours=(data[19] << 16) + data[20];
	printf("hours = %d h\n",hours);
	
	ah_total=((data[21] << 16) + data[22])*0.1;
	printf("ah_total = %.2f Ah\n",ah_total);
	
	ah_b1=((data[23] << 16) + data[24])*0.1;
	printf("ah_b1 = %.2f Ah\n",ah_b1);
	
	ah_b2=((data[25] << 16) + data[26])*0.1;
	printf("ah_b2 = %.2f Ah\n",ah_b2);
	
	printf("\nBattery 1 Present Configuration\n\n");
	
	tcompcoef=data[27]/1800.0;
	printf("tcompcoef=%.2f V\n",tcompcoef);
	
	priority=data[28]*100.0/417.0;
	printf("priority=%.2f %%\n",priority);
	
	vabs1=data[29]/1800.0;
	printf("vabs1=%.2f V\n",vabs1);
	
	veql1=data[30]/1800.0;
	printf("veql1=%.2f V\n",veql1);
	
	vfloat1=data[31]/1800.0;
	printf("vfloat1=%.2f V\n",vfloat1);
	
	vclamp1=data[32]/1800.0;
	printf("vclamp1=%.2f V\n",vclamp1);
	
	tfloat1=data[33];
	printf("tfloat1 = %d min\n",tfloat1);
	
	teq1=data[34];
	printf("teq1 = %d min\n",teq1);
	
	printf("\nBattery 2 Present Configuration\n\n");
	
	vabs2=data[35]/1800.0;
	printf("vabs2=%.2f V\n",vabs2);
	
	veql2=data[36]/1800.0;
	printf("veql2=%.2f V\n",veql2);
	
	vfloat2=data[37]/1800.0;
	printf("vfloat2=%.2f V\n",vfloat2);
	
	vclamp2=data[38]/1800.0;
	printf("vclamp2=%.2f V\n",vclamp2);
	
	tfloat2=data[39];
	printf("tfloat2 = %d min\n",tfloat2);
	
	teq2=data[40];
	printf("teq2 = %d min\n",teq2);
	
	usleep(2500);		// Give the charge controller time before requesting next set of registers
	
	/* Read the last 17 RAM Registers */
	rc = modbus_read_registers(ctx, 0x0100, 17, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	printf("\nPre-scaled Values\n\n");
	
	ta_F=data[0];
	printf("ta_F = %d °F\n",ta_F);
	
	ta_C=data[1];
	printf("ta_C = %d °C\n",ta_C);
	
	tr_F=data[2];
	printf("tr_F = %d °F\n",tr_F);
	
	tr_C=data[3];
	printf("tr_C = %d °C\n",tr_C);
	
	ths_F=data[4];
	printf("ths_F = %d °F\n",ths_F);
	
	ths_C=data[5];
	printf("ths_C = %d °C\n",ths_C);
	
	dc1=data[6];
	printf("dc1 = %d %%\n",dc1);
	
	dc2=data[7];
	printf("dc2 = %d %%\n",dc2);
	
	state=data[8];
	switch (state) {
		case 1:
			printf("state = %d - NORMAL\n",state);
			break;
		case 3:
			printf("state = %d - NIGHT\n",state);
			break;
		case 4:
			printf("state = %d - FAULT\n",state);
			break;
	}
	
	faults=data[9];
	printf("faults = Self-diagnostic faults:\n");
	if (faults == 0) {
		printf("\tNo faults\n");
	} else {
		if (faults & 1) printf("\tReverse polarity solar connection\n");
		if ((faults & (1 << 1)) >> 1) printf("\tReverse polarity battery 1\n");
		if ((faults & (1 << 2)) >> 2) printf("\tReverse polarity battery 2\n");
		if ((faults & (1 << 3)) >> 3) printf("\tDamaged local temperature sensor\n");
		if ((faults & (1 << 4)) >> 4) printf("\tDamaged or removed RTS\n");
		if ((faults & (1 << 5)) >> 5) printf("\tOver-current condition\n");
		if ((faults & (1 << 6)) >> 6) printf("\tHigh Temperature disconnect\n");
		if ((faults & (1 << 7)) >> 7) printf("\tHigh Voltage disconnect\n");
	}
	
	flags=data[10];
	printf("flags = Self-diagnostic faults:\n");
	if (flags == 0) {
		printf("\tNo flags\n");
	} else {
		if (flags & 1) printf("\tinternal use\n");
		if ((flags & (1 << 1)) >> 1) printf("\tinternal use\n");
		if ((flags & (1 << 2)) >> 2) printf("\tinternal use\n");
		if ((flags & (1 << 3)) >> 3) printf("\tRegulation\n");
		if ((flags & (1 << 4)) >> 4) printf("\tValid RTS\n");
		if ((flags & (1 << 5)) >> 5) printf("\tAlternating\n");
		if ((flags & (1 << 6)) >> 6) printf("\tOn / Off regulation\n");
		if ((flags & (1 << 7)) >> 7) printf("\tStarted\n");
	}
	
	DIPs=data[11];
	printf("DIPs = DIP switch settings:\n");
	if (DIPs & 1) {
		printf("\tSwitch 1 ON - Battery 1 Type: Flooded\n");
	} else {
		printf("\tSwitch 1 OFF - Battery 1 Type: Sealed\n");
	}
	if ((DIPs & (1 << 1)) >> 1) {
		printf("\tSwitch 2 ON - Battery 2 Type: Flooded\n");
	} else {
		printf("\tSwitch 2 OFF - Battery 2 Type: Sealed\n");
	}
	if ((DIPs & (1 << 2)) >> 2) {
		printf("\tSwitch 3 ON - Battery Charging Priority - 50%% / 50%% Priority\n");
	} else {
		printf("\tSwitch 3 OFF - Battery Charging Priority - 90%% / 10%% Priority\n");
	}
	if ((DIPs & (1 << 3)) >> 3) {
		printf("\tSwitch 4 ON - Custom Setpoints - Custom programmed settings\n");
	} else {
		printf("\tSwitch 4 OFF - Custom Setpoints - Factory settings\n");
	}
	if ((DIPs & (1 << 4)) >> 4) {
		printf("\tSwitch 5 ON - Regulation Type - Slow Switching Regulation\n");
	} else {
		printf("\tSwitch 5 OFF - Regulation Type - PWM Regulation\n");
	}
	
	b1_state=data[13];
	switch (b1_state) {
		case 0:
			printf("b1_state = %d - NORMAL\n",b1_state);
			break;
		case 1:
			printf("b1_state = %d - EQUALIZE\n",b1_state);
			break;
		case 2:
			printf("b1_state = %d - FLOAT\n",b1_state);
			break;
	}
	
	b2_state=data[14];
	switch (b2_state) {
		case 0:
			printf("b2_state = %d - NORMAL\n",b2_state);
			break;
		case 1:
			printf("b2_state = %d - EQUALIZE\n",b2_state);
			break;
		case 2:
			printf("b2_state = %d - FLOAT\n",b2_state);
			break;
	}
	
	b1_detect=data[15];
	switch (b1_detect) {
		case 0:
			printf("b1_detect = %d - DISCONNECTED\n",b1_detect);
			break;
		case 1:
			printf("b1_detect = %d - CONNECTED\n",b1_detect);
			break;
	}
	
	b2_detect=data[16];
	switch (b2_detect) {
		case 0:
			printf("b2_detect = %d - DISCONNECTED\n",b2_detect);
			break;
		case 1:
			printf("b2_detect = %d - CONNECTED\n",b2_detect);
			break;
	}
	
	/* Close the MODBUS connection */
    modbus_close(ctx);
    modbus_free(ctx);
	
	return(0);
}

